/**
 * Fichero : Grafo.h      Fecha:08/03/2019
 * Autores: 
 *
 *  Fichero de cabecera con estructuras y prototipos 
 *  para implementar el TAD Grafo.
 *
 * (C) 2016-2019 Escuela Politecnica Superior, Universidad Autonoma de Madrid
 */

#ifndef __GRAFO_H__
#define __GRAFO_H__

/********************************************** Estructuras del TAD Grafo ***/

typedef struct _arista {
  int dest;          /* Indice del vertice de destino*/
  int peso;          /* Ponderacion de la arista*/
  struct _arista *sig; /* Puntero a la siguiente arista*/
} Arista;

typedef struct {
  int  id;           /* Numero de vertice*/
  Arista *aristas;   /* Lista enlazada con las aristas*/
} Vertice;

typedef struct {  
  int n_vertices;     /* Numero de vertices del Grafo*/
  int n_aristas;      /* Numero de aristas del Grafo*/
  int es_dirigido;    /* Indicador si el grafo es dirigido */
  Vertice *vertices;  /* Tabla de vertices*/
} Grafo;


/*
 *  Funcion para crear un Grafo vacio con n_vertices 
 */
Grafo *crearGrafo(int n_vertices);
/*
 *  Funcion para crear un Grafo dirigido vacio con n_vertices. Para los 
 *  grafos dirigidos, cada vez que se aniada una arista nueva al grafo se
 *  incluira tanto en la lista de adyacencia del vertice origen y destino 
 */
Grafo *crearGrafoDirigido(int n_vertices);

/*
 * Aniade una arista al Grafo. Si el grafo es dirigido se incluirá la arista
 * como orig->dest y como dest->orig
 */
void aniadirArista(Grafo *g, int orig, int dest, int peso);

/*
 *  Libera todo la memoria reservada para el Grafo 
 */
void borrarGrafo(Grafo *g);

/*
 *  Devuelve si el grafo g es dirigido
 */
int esDirigido(Grafo *g);

/*
 *  Imprime el Grafo 'g'
 */
void imprimirGrafo(Grafo *g);

/*
 * Implementar una función que realice un recorrido en anchura. Esta función
 * recibe como argumento un grafo y debe imprimir las aristas en el orden dado
 * por un recorrido en anchura. 
 */
void imprimirEnAnchura(Grafo *g, int partida);

/*
 * Implementar una función que realice un recorrido en profundidad. Esta función
 * recibe como argumento un grafo y debe imprimir las aristas en el orden dado
 * por un recorrido en profundidad.
 */
void imprimirEnProfundidad(Grafo *g, int partida);

/* Función que cuente el número de arcos de entrada y de salida para cada
 * vértice del grafo g.
 *
 * Salida:
 *   Array de de dimensión 2*n_vertices que contiene en la posición 2i y 2i+1 el
 *   número de arcos de salida y de entrada respectivamente para el vértice i,
 *   con i = 1...n_vertices
 *
 */
int *contarGradosVertices(Grafo *g);

/*
 * Función que realiza el recorrido en orden topológico del grafo g.
 *
 * Salida:
 *   Si es posible hacer el recorrido completo del grafo, la función debe
 *   devolver 1 y si no 0.
 */
int imprimirOrdenacionTopologica(Grafo *g);

/*
 * Funcion que genera un Grafo conexo regular con 2*n_vertices=n_aristas
 *   en el que cada vertice esta unido a sus 4 vecinos mas proximos. La 
 *   poderacion de las aristas es aleatoria y va de 1 a 100.
 *
 */
Grafo *crearGrafoRegular(int n_vertices);

/*
 * Genera un grafo dirigido aleatorio de n_vertices y n_aristas  con una  
 *   ponderacion de las aristas entre min_pond y max_pond
 */
Grafo *crearGrafoAleatorio(int n_vertices, int n_aristas, int min_pond, int max_pond);

#endif
